@@ -8,7 +8,7 @@
 */
 
 #include "prprf.h"
-#include "prlog.h"
+#include "mozilla/Logging.h"
 #include "msgCore.h"
 #include "nsMsgMaildirStore.h"
 #include "nsIMsgFolder.h"
@@ -33,6 +33,7 @@
 #include "nsITimer.h"
 #include "nsIMailboxUrl.h"
 #include "nsIMsgMailNewsUrl.h"
+#include "nsIMsgFilterPlugin.h"
 #include "nsLocalUndoTxn.h"
 #include "nsIMessenger.h"
 
@@ -47,7 +48,7 @@ nsMsgMaildirStore::~nsMsgMaildirStore()
 {
 }
 
-NS_IMPL_ISUPPORTS1(nsMsgMaildirStore, nsIMsgPluggableStore)
+NS_IMPL_ISUPPORTS(nsMsgMaildirStore, nsIMsgPluggableStore)
 
 // Iterates over the folders in the "path" directory, and adds subfolders to
 // parent for each Maildir folder found.
@@ -80,8 +81,8 @@ nsresult nsMsgMaildirStore::AddSubFolders(nsIMsgFolder *parent, nsIFile *path,
   }
 
   // add the folders
-  PRInt32 count = currentDirEntries.Count();
-  for (PRInt32 i = 0; i < count; ++i)
+  int32_t count = currentDirEntries.Count();
+  for (int32_t i = 0; i < count; ++i)
   {
     nsCOMPtr<nsIFile> currentFile(currentDirEntries[i]);
 
@@ -138,18 +139,22 @@ NS_IMETHODIMP nsMsgMaildirStore::DiscoverSubFolders(nsIMsgFolder *aParentFolder,
 }
 
 /**
- *Create a Maildir-style folder with "tmp", " and "cur" subfolders
- * but no "new" subfolder, because it's not sensical in the mail client context.
- ("new" directory is for messages on the server that haven't been seen by a
-*  mail client).
- * aFolderName is already "safe" - it has been through NS_MsgHashIfNecessary
+ * Create if missing a Maildir-style folder with "tmp" and "cur" subfolders
+ * but no "new" subfolder, because it doesn't make sense in the mail client
+ * context. ("new" directory is for messages on the server that haven't been
+*  seen by a mail client).
+ * aFolderName is already "safe" - it has been through NS_MsgHashIfNecessary.
  */
 nsresult nsMsgMaildirStore::CreateMaildir(nsIFile *path)
 {
   nsresult rv = path->Create(nsIFile::DIRECTORY_TYPE, 0700);
-  NS_ENSURE_SUCCESS(rv, rv);
+  if (NS_FAILED(rv) && rv != NS_ERROR_FILE_ALREADY_EXISTS)
+  {
+    NS_WARNING("Could not create root directory for message folder");
+    return rv;
+  }
 
-  // Create tmp, new, cur leaves
+  // Create tmp, cur leaves
   nsCOMPtr<nsIFile> leaf(do_CreateInstance(NS_LOCAL_FILE_CONTRACTID, &rv));
   NS_ENSURE_SUCCESS(rv, rv);
 
@@ -157,11 +162,19 @@ nsresult nsMsgMaildirStore::CreateMaildir(nsIFile *path)
 
   leaf->AppendNative(NS_LITERAL_CSTRING("tmp"));
   rv = leaf->Create(nsIFile::DIRECTORY_TYPE, 0700);
-  NS_ENSURE_SUCCESS(rv, rv);
+  if (NS_FAILED(rv) && rv != NS_ERROR_FILE_ALREADY_EXISTS)
+  {
+    NS_WARNING("Could not create tmp directory for message folder");
+    return rv;
+  }
 
   leaf->SetNativeLeafName(NS_LITERAL_CSTRING("cur"));
   rv = leaf->Create(nsIFile::DIRECTORY_TYPE, 0700);
-  NS_ENSURE_SUCCESS(rv, rv);
+  if (NS_FAILED(rv) && rv != NS_ERROR_FILE_ALREADY_EXISTS)
+  {
+    NS_WARNING("Could not create cur directory for message folder");
+    return rv;
+  }
 
   return NS_OK;
 }
@@ -170,6 +183,11 @@ NS_IMETHODIMP nsMsgMaildirStore::CreateFolder(nsIMsgFolder *aParent,
                                               const nsAString &aFolderName,
                                               nsIMsgFolder **aResult)
 {
+  NS_ENSURE_ARG_POINTER(aParent);
+  NS_ENSURE_ARG_POINTER(aResult);
+  if (aFolderName.IsEmpty())
+    return NS_MSG_ERROR_INVALID_FOLDER_NAME;
+
   nsCOMPtr <nsIFile> path;
   nsresult rv = aParent->GetFilePath(getter_AddRefs(path));
   NS_ENSURE_SUCCESS(rv, rv);
@@ -185,6 +203,10 @@ NS_IMETHODIMP nsMsgMaildirStore::CreateFolder(nsIMsgFolder *aParent,
   NS_MsgHashIfNecessary(safeFolderName);
 
   path->Append(safeFolderName);
+  bool exists;
+  path->Exists(&exists);
+  if (exists) //check this because localized names are different from disk names
+    return NS_MSG_FOLDER_EXISTS;
 
   rv = CreateMaildir(path);
   NS_ENSURE_SUCCESS(rv, rv);
@@ -224,7 +246,7 @@ NS_IMETHODIMP nsMsgMaildirStore::CreateFolder(nsIMsgFolder *aParent,
     }
     else
     {
-      PR_LOG(MailDirLog, PR_LOG_ALWAYS,
+      MOZ_LOG(MailDirLog, mozilla::LogLevel::Info,
             ("CreateFolder - failed creating db for new folder\n"));
       path->Remove(true); // recursive
       rv = NS_MSG_CANT_CREATE_FOLDER;
@@ -235,13 +257,20 @@ NS_IMETHODIMP nsMsgMaildirStore::CreateFolder(nsIMsgFolder *aParent,
 }
 
 NS_IMETHODIMP nsMsgMaildirStore::HasSpaceAvailable(nsIMsgFolder *aFolder,
-                                                   PRInt64 aSpaceRequested,
+                                                   int64_t aSpaceRequested,
                                                    bool *aResult)
 {
-  // This should probably check disk space available, though the caller
-  // might be doing that as well.
   NS_ENSURE_ARG_POINTER(aResult);
-  *aResult = true;
+  NS_ENSURE_ARG_POINTER(aFolder);
+
+  nsCOMPtr<nsIFile> pathFile;
+  nsresult rv = aFolder->GetFilePath(getter_AddRefs(pathFile));
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  *aResult = DiskSpaceAvailableInStore(pathFile, aSpaceRequested);
+  if (!*aResult)
+    return NS_ERROR_FILE_DISK_FULL;
+
   return NS_OK;
 }
 
@@ -270,7 +299,7 @@ NS_IMETHODIMP nsMsgMaildirStore::IsSummaryFileValid(nsIMsgFolder *aFolder,
     newFile->Exists(&exists);
     if (!exists)
     {
-      PRInt32 numMessages;
+      int32_t numMessages;
       dbFolderInfo->GetNumMessages(&numMessages);
       if (!numMessages)
         *aResult = true;
@@ -287,38 +316,10 @@ NS_IMETHODIMP nsMsgMaildirStore::SetSummaryFileValid(nsIMsgFolder *aFolder,
   NS_ENSURE_ARG_POINTER(aDB);
   nsCOMPtr<nsIDBFolderInfo> dbFolderInfo;
   aDB->GetDBFolderInfo(getter_AddRefs(dbFolderInfo));
+  NS_ENSURE_STATE(dbFolderInfo);
   return dbFolderInfo->SetBooleanProperty("maildirValid", aValid);
 }
 
-NS_IMETHODIMP nsMsgMaildirStore::GetSummaryFile(nsIMsgFolder *aFolder,
-                                                nsIFile **aSummaryFile)
-{
-  NS_ENSURE_ARG_POINTER(aFolder);
-  NS_ENSURE_ARG_POINTER(aSummaryFile);
-
-  nsresult rv;
-  nsCOMPtr <nsIFile> newSummaryLocation =
-    do_CreateInstance(NS_LOCAL_FILE_CONTRACTID, &rv);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  nsCOMPtr<nsIFile> pathFile;
-  rv = aFolder->GetFilePath(getter_AddRefs(pathFile));
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  newSummaryLocation->InitWithFile(pathFile);
-  nsString fileName;
-
-  rv = newSummaryLocation->GetLeafName(fileName);
-  if (NS_FAILED(rv))
-    return rv;
-  fileName.Append(NS_LITERAL_STRING(SUMMARY_SUFFIX));
-  rv = newSummaryLocation->SetLeafName(fileName);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  NS_IF_ADDREF(*aSummaryFile = newSummaryLocation);
-  return NS_OK;
-}
-
 NS_IMETHODIMP nsMsgMaildirStore::DeleteFolder(nsIMsgFolder *aFolder)
 {
   NS_ENSURE_ARG_POINTER(aFolder);
@@ -351,7 +352,7 @@ NS_IMETHODIMP nsMsgMaildirStore::RenameFolder(nsIMsgFolder *aFolder,
 
   // old sbd directory
   nsCOMPtr<nsIFile> sbdPathFile;
-  PRUint32 numChildren;
+  uint32_t numChildren;
   aFolder->GetNumSubFolders(&numChildren);
   if (numChildren > 0)
   {
@@ -364,7 +365,7 @@ NS_IMETHODIMP nsMsgMaildirStore::RenameFolder(nsIMsgFolder *aFolder,
 
   // old summary
   nsCOMPtr<nsIFile> oldSummaryFile;
-  rv = GetSummaryFile(aFolder, getter_AddRefs(oldSummaryFile));
+  rv = aFolder->GetSummaryFile(getter_AddRefs(oldSummaryFile));
   NS_ENSURE_SUCCESS(rv, rv);
 
   // Validate new name
@@ -374,7 +375,7 @@ NS_IMETHODIMP nsMsgMaildirStore::RenameFolder(nsIMsgFolder *aFolder,
   aFolder->ForceDBClosed();
 
   // rename folder
-  rv = oldPathFile->MoveTo(nsnull, safeName);
+  rv = oldPathFile->MoveTo(nullptr, safeName);
   NS_ENSURE_SUCCESS(rv, rv);
 
   if (numChildren > 0)
@@ -382,12 +383,13 @@ NS_IMETHODIMP nsMsgMaildirStore::RenameFolder(nsIMsgFolder *aFolder,
     // rename "*.sbd" directory
     nsAutoString sbdName = safeName;
     sbdName += NS_LITERAL_STRING(FOLDER_SUFFIX);
-    sbdPathFile->MoveTo(nsnull, sbdName);
+    sbdPathFile->MoveTo(nullptr, sbdName);
   }
 
   // rename summary
-  safeName += NS_LITERAL_STRING(SUMMARY_SUFFIX);
-  oldSummaryFile->MoveTo(nsnull, safeName);
+  nsAutoString summaryName(safeName);
+  summaryName += NS_LITERAL_STRING(SUMMARY_SUFFIX);
+  oldSummaryFile->MoveTo(nullptr, summaryName);
 
   nsCOMPtr<nsIMsgFolder> parentFolder;
   rv = aFolder->GetParent(getter_AddRefs(parentFolder));
@@ -401,12 +403,18 @@ NS_IMETHODIMP nsMsgMaildirStore::CopyFolder(nsIMsgFolder *aSrcFolder,
                                             nsIMsgFolder *aDstFolder,
                                             bool aIsMoveFolder,
                                             nsIMsgWindow *aMsgWindow,
-                                            nsIMsgCopyServiceListener *aListener)
+                                            nsIMsgCopyServiceListener *aListener,
+                                            const nsAString &aNewName)
 {
   NS_ENSURE_ARG_POINTER(aSrcFolder);
   NS_ENSURE_ARG_POINTER(aDstFolder);
-  nsString folderName;
-  aSrcFolder->GetName(folderName);
+
+  nsAutoString folderName;
+  if (aNewName.IsEmpty())
+    aSrcFolder->GetName(folderName);
+  else
+    folderName.Assign(aNewName);
+
   nsAutoString safeFolderName(folderName);
   NS_MsgHashIfNecessary(safeFolderName);
   nsCOMPtr<nsIMsgLocalMailFolder> localSrcFolder(do_QueryInterface(aSrcFolder));
@@ -432,19 +440,21 @@ NS_IMETHODIMP nsMsgMaildirStore::CopyFolder(nsIMsgFolder *aSrcFolder,
   nsCOMPtr<nsIFile> origPath;
   oldPath->Clone(getter_AddRefs(origPath));
 
-  rv = oldPath->CopyTo(newPath, EmptyString());
+  rv = oldPath->CopyTo(newPath, safeFolderName);
   NS_ENSURE_SUCCESS(rv, rv); //will fail if a file by that name exists
 
   // Copy to dir can fail if file does not exist. If copy fails, we test
   // if the file exists or not, if it does not that's ok, we continue
   // without copying it. If it fails and file exist and is not zero sized
   // there is real problem.
-  rv = summaryFile->CopyTo(newPath, EmptyString());
+  nsAutoString dbName(safeFolderName);
+  dbName += NS_LITERAL_STRING(SUMMARY_SUFFIX);
+  rv = summaryFile->CopyTo(newPath, dbName);
   if (!NS_SUCCEEDED(rv))
   {
     // Test if the file is not empty
     bool exists;
-    PRInt64 fileSize;
+    int64_t fileSize;
     summaryFile->Exists(&exists);
     summaryFile->GetFileSize(&fileSize);
     if (exists && fileSize > 0)
@@ -459,7 +469,7 @@ NS_IMETHODIMP nsMsgMaildirStore::CopyFolder(nsIMsgFolder *aSrcFolder,
   NS_ENSURE_SUCCESS(rv, rv);
 
   newMsgFolder->SetPrettyName(folderName);
-  PRUint32 flags;
+  uint32_t flags;
   aSrcFolder->GetFlags(&flags);
   newMsgFolder->SetFlags(flags);
   bool changed = false;
@@ -517,7 +527,7 @@ NS_IMETHODIMP nsMsgMaildirStore::CopyFolder(nsIMsgFolder *aSrcFolder,
 
     nsCOMPtr<nsIMsgFolder> msgParent;
     aSrcFolder->GetParent(getter_AddRefs(msgParent));
-    aSrcFolder->SetParent(nsnull);
+    aSrcFolder->SetParent(nullptr);
     if (msgParent)
     {
       // The files have already been moved, so delete storage false
@@ -550,7 +560,7 @@ NS_IMETHODIMP nsMsgMaildirStore::CopyFolder(nsIMsgFolder *aSrcFolder,
       nsCOMPtr<nsIMsgFolder> msgParent;
       newMsgFolder->ForceDBClosed();
       newMsgFolder->GetParent(getter_AddRefs(msgParent));
-      newMsgFolder->SetParent(nsnull);
+      newMsgFolder->SetParent(nullptr);
       if (msgParent)
       {
         msgParent->PropagateDelete(newMsgFolder, false, aMsgWindow);
@@ -602,18 +612,19 @@ nsMsgMaildirStore::GetNewMsgOutputStream(nsIMsgFolder *aFolder,
   bool exists;
   newFile->Exists(&exists);
   if (!exists) {
-    PR_LOG(MailDirLog, PR_LOG_ALWAYS,
+    MOZ_LOG(MailDirLog, mozilla::LogLevel::Info,
            ("GetNewMsgOutputStream - tmp subfolder does not exist!!\n"));
     rv = newFile->Create(nsIFile::DIRECTORY_TYPE, 0755);
     NS_ENSURE_SUCCESS(rv, rv);
   }
 
   // generate new file name
-  nsCAutoString newName;
-  newName.AppendInt(PR_Now());
+  nsAutoCString newName;
+  newName.AppendInt(static_cast<int64_t>(PR_Now()));
   newFile->AppendNative(newName);
   // CreateUnique, in case we get more than one message per millisecond :-)
-  newFile->CreateUnique(nsIFile::NORMAL_FILE_TYPE, 0600);
+  rv = newFile->CreateUnique(nsIFile::NORMAL_FILE_TYPE, 0600);
+  NS_ENSURE_SUCCESS(rv, rv);
   newFile->GetNativeLeafName(newName);
   // save the file name in the message header - otherwise no way to retrieve it
   (*aNewMsgHdr)->SetStringProperty("storeToken", newName.get());
@@ -630,7 +641,7 @@ nsMsgMaildirStore::DiscardNewMessage(nsIOutputStream *aOutputStream,
 
   aOutputStream->Close();
   // file path is stored in message header property "storeToken"
-  nsCAutoString fileName;
+  nsAutoCString fileName;
   aNewHdr->GetStringProperty("storeToken", getter_Copies(fileName));
   if (fileName.IsEmpty())
     return NS_ERROR_FAILURE;
@@ -666,7 +677,7 @@ nsMsgMaildirStore::FinishNewMessage(nsIOutputStream *aOutputStream,
   NS_ENSURE_SUCCESS(rv, rv);
 
   // file path is stored in message header property
-  nsCAutoString fileName;
+  nsAutoCString fileName;
   aNewHdr->GetStringProperty("storeToken", getter_Copies(fileName));
   if (fileName.IsEmpty())
   {
@@ -674,6 +685,20 @@ nsMsgMaildirStore::FinishNewMessage(nsIOutputStream *aOutputStream,
     return NS_ERROR_FAILURE;
   }
 
+  // path to the new destination
+  nsCOMPtr<nsIFile> toPath;
+  folderPath->Clone(getter_AddRefs(toPath));
+  toPath->Append(NS_LITERAL_STRING("cur"));
+
+  // let's check if the folder exists
+  bool exists;
+  toPath->Exists(&exists);
+  if (!exists)
+  {
+    rv = toPath->Create(nsIFile::DIRECTORY_TYPE, 0755);
+    NS_ENSURE_SUCCESS(rv, rv);
+  }
+
   // path to the downloaded message
   nsCOMPtr<nsIFile> fromPath;
   folderPath->Clone(getter_AddRefs(fromPath));
@@ -681,49 +706,53 @@ nsMsgMaildirStore::FinishNewMessage(nsIOutputStream *aOutputStream,
   fromPath->AppendNative(fileName);
 
   // let's check if the tmp file exists
-  bool exists;
   fromPath->Exists(&exists);
   if (!exists)
   {
+    // Perhaps the message has already moved. See bug 1028372 to fix this.
+    toPath->AppendNative(fileName);
+    toPath->Exists(&exists);
+    if (exists) // then there is nothing to do
+      return NS_OK;
+
     NS_ERROR("FinishNewMessage - oops! file does not exist!");
-    return NS_ERROR_FAILURE;
+    return NS_ERROR_FILE_TARGET_DOES_NOT_EXIST;
   }
 
-  // move to the "cur" subfolder
-  nsCOMPtr<nsIFile> toPath;
-  folderPath->Clone(getter_AddRefs(toPath));
-  toPath->Append(NS_LITERAL_STRING("cur"));
+  nsCOMPtr<nsIFile> existingPath;
+  toPath->Clone(getter_AddRefs(existingPath));
+  existingPath->AppendNative(fileName);
+  existingPath->Exists(&exists);
 
-  // let's check if the folder exists
-  toPath->Exists(&exists);
-  if (!exists)
-  {
-    rv = toPath->Create(nsIFile::DIRECTORY_TYPE, 0755);
+  if (exists) {
+    rv = existingPath->CreateUnique(nsIFile::NORMAL_FILE_TYPE, 0600);
     NS_ENSURE_SUCCESS(rv, rv);
+    existingPath->GetNativeLeafName(fileName);
+    aNewHdr->SetStringProperty("storeToken", fileName.get());
   }
 
   return fromPath->MoveToNative(toPath, fileName);
 }
 
 NS_IMETHODIMP
-nsMsgMaildirStore::MoveNewlyDownloadedMessage(nsIMsgDBHdr *aNewHdr,
+nsMsgMaildirStore::MoveNewlyDownloadedMessage(nsIMsgDBHdr *aHdr,
                                               nsIMsgFolder *aDestFolder,
                                               bool *aResult)
 {
-  NS_ENSURE_ARG_POINTER(aNewHdr);
+  NS_ENSURE_ARG_POINTER(aHdr);
   NS_ENSURE_ARG_POINTER(aDestFolder);
   NS_ENSURE_ARG_POINTER(aResult);
 
   nsCOMPtr<nsIFile> folderPath;
   nsCOMPtr<nsIMsgFolder> folder;
-  nsresult rv = aNewHdr->GetFolder(getter_AddRefs(folder));
+  nsresult rv = aHdr->GetFolder(getter_AddRefs(folder));
   NS_ENSURE_SUCCESS(rv, rv);
   rv = folder->GetFilePath(getter_AddRefs(folderPath));
   NS_ENSURE_SUCCESS(rv, rv);
 
   // file path is stored in message header property
-  nsCAutoString fileName;
-  aNewHdr->GetStringProperty("storeToken", getter_Copies(fileName));
+  nsAutoCString fileName;
+  aHdr->GetStringProperty("storeToken", getter_Copies(fileName));
   if (fileName.IsEmpty())
   {
     NS_ERROR("FinishNewMessage - no storeToken in msg hdr!!\n");
@@ -759,18 +788,99 @@ nsMsgMaildirStore::MoveNewlyDownloadedMessage(nsIMsgDBHdr *aNewHdr,
     NS_ENSURE_SUCCESS(rv, rv);
   }
 
+  nsCOMPtr<nsIMsgDatabase> destMailDB;
+  rv = aDestFolder->GetMsgDatabase(getter_AddRefs(destMailDB));
+  NS_WARNING_ASSERTION(destMailDB && NS_SUCCEEDED(rv),
+                       "failed to open mail db moving message");
+
+  nsCOMPtr<nsIMsgDBHdr> newHdr;
+  if (destMailDB)
+    rv = destMailDB->CopyHdrFromExistingHdr(nsMsgKey_None, aHdr, true,
+                                            getter_AddRefs(newHdr));
+  if (NS_SUCCEEDED(rv) && !newHdr)
+    rv = NS_ERROR_UNEXPECTED;
+
+  if (NS_FAILED(rv))
+    aDestFolder->ThrowAlertMsg("filterFolderHdrAddFailed", nullptr);
+
+  nsCOMPtr<nsIFile> existingPath;
+  toPath->Clone(getter_AddRefs(existingPath));
+  existingPath->AppendNative(fileName);
+  existingPath->Exists(&exists);
+
+  if (exists) {
+    rv = existingPath->CreateUnique(nsIFile::NORMAL_FILE_TYPE, 0600);
+    NS_ENSURE_SUCCESS(rv, rv);
+    existingPath->GetNativeLeafName(fileName);
+    newHdr->SetStringProperty("storeToken", fileName.get());
+  }
+
   rv = fromPath->MoveToNative(toPath, fileName);
   *aResult = NS_SUCCEEDED(rv);
+  if (NS_FAILED(rv))
+    aDestFolder->ThrowAlertMsg("filterFolderWriteFailed", nullptr);
+
+  if (NS_FAILED(rv)) {
+    if (destMailDB)
+      destMailDB->Close(true);
+
+    return NS_MSG_ERROR_WRITING_MAIL_FOLDER;
+  }
+
+  bool movedMsgIsNew = false;
+  // if we have made it this far then the message has successfully been
+  // written to the new folder now add the header to the destMailDB.
+
+  uint32_t newFlags;
+  newHdr->GetFlags(&newFlags);
+  nsMsgKey msgKey;
+  newHdr->GetMessageKey(&msgKey);
+  if (!(newFlags & nsMsgMessageFlags::Read))
+  {
+    nsCString junkScoreStr;
+    (void) newHdr->GetStringProperty("junkscore", getter_Copies(junkScoreStr));
+    if (atoi(junkScoreStr.get()) != nsIJunkMailPlugin::IS_SPAM_SCORE) {
+      newHdr->OrFlags(nsMsgMessageFlags::New, &newFlags);
+      destMailDB->AddToNewList(msgKey);
+      movedMsgIsNew = true;
+    }
+  }
+
+  nsCOMPtr<nsIMsgFolderNotificationService> notifier(
+    do_GetService(NS_MSGNOTIFICATIONSERVICE_CONTRACTID));
+  if (notifier)
+    notifier->NotifyMsgAdded(newHdr);
+
+  if (movedMsgIsNew) {
+    aDestFolder->SetHasNewMessages(true);
+
+    // Notify the message was moved.
+    if (notifier) {
+      notifier->NotifyItemEvent(folder,
+                                NS_LITERAL_CSTRING("UnincorporatedMessageMoved"),
+                                newHdr);
+    }
+  }
+
+  nsCOMPtr<nsIMsgDatabase> sourceDB;
+  rv = folder->GetMsgDatabase(getter_AddRefs(sourceDB));
+
+  if (NS_SUCCEEDED(rv) && sourceDB)
+    sourceDB->RemoveHeaderMdbRow(aHdr);
+
+  destMailDB->SetSummaryValid(true);
+  aDestFolder->UpdateSummaryTotals(true);
+  destMailDB->Commit(nsMsgDBCommitType::kLargeCommit);
   return rv;
 }
 
 NS_IMETHODIMP
 nsMsgMaildirStore::GetMsgInputStream(nsIMsgFolder *aMsgFolder,
                                      const nsACString &aMsgToken,
-                                     PRInt64 *aOffset,
+                                     int64_t *aOffset,
                                      nsIMsgDBHdr *aMsgHdr,
-                                     bool *aReusable NS_OUTPARAM,
-                                     nsIInputStream **aResult NS_OUTPARAM)
+                                     bool *aReusable,
+                                     nsIInputStream **aResult)
 {
   NS_ENSURE_ARG_POINTER(aMsgFolder);
   NS_ENSURE_ARG_POINTER(aOffset);
@@ -786,35 +896,35 @@ nsMsgMaildirStore::GetMsgInputStream(nsIMsgFolder *aMsgFolder,
 
   if (aMsgToken.IsEmpty())
   {
-    PR_LOG(MailDirLog, PR_LOG_ALWAYS,
+    MOZ_LOG(MailDirLog, mozilla::LogLevel::Info,
            ("GetMsgInputStream - empty storeToken!!\n"));
     return NS_ERROR_FAILURE;
   }
 
   path->Append(NS_LITERAL_STRING("cur"));
-  path->AppendNative(aMsgToken);
 
   // let's check if the folder exists
   bool exists;
   path->Exists(&exists);
   if (!exists) {
-    PR_LOG(MailDirLog, PR_LOG_ALWAYS,
+    MOZ_LOG(MailDirLog, mozilla::LogLevel::Info,
            ("GetMsgInputStream - oops! cur subfolder does not exist!\n"));
     rv = path->Create(nsIFile::DIRECTORY_TYPE, 0755);
     NS_ENSURE_SUCCESS(rv, rv);
   }
 
+  path->AppendNative(aMsgToken);
   return NS_NewLocalFileInputStream(aResult, path);
 }
 
 NS_IMETHODIMP nsMsgMaildirStore::DeleteMessages(nsIArray *aHdrArray)
 {
-  PRUint32 messageCount;
+  uint32_t messageCount;
   nsresult rv = aHdrArray->GetLength(&messageCount);
   NS_ENSURE_SUCCESS(rv, rv);
   nsCOMPtr<nsIMsgFolder> folder;
 
-  for (PRUint32 i = 0; i < messageCount; i++)
+  for (uint32_t i = 0; i < messageCount; i++)
   {
     nsCOMPtr<nsIMsgDBHdr> msgHdr = do_QueryElementAt(aHdrArray, i, &rv);
     if (NS_FAILED(rv))
@@ -823,24 +933,30 @@ NS_IMETHODIMP nsMsgMaildirStore::DeleteMessages(nsIArray *aHdrArray)
     nsCOMPtr<nsIFile> path;
     rv = folder->GetFilePath(getter_AddRefs(path));
     NS_ENSURE_SUCCESS(rv, rv);
-    nsCAutoString fileName;
+    nsAutoCString fileName;
     msgHdr->GetStringProperty("storeToken", getter_Copies(fileName));
-    if (fileName.IsEmpty())
-      return NS_ERROR_FAILURE;
 
     if (fileName.IsEmpty())
     {
-      PR_LOG(MailDirLog, PR_LOG_ALWAYS,
-             ("GetMsgInputStream - empty storeToken!!\n"));
-      return NS_ERROR_FAILURE;
+      MOZ_LOG(MailDirLog, mozilla::LogLevel::Info,
+             ("DeleteMessages - empty storeToken!!\n"));
+      // Perhaps an offline store has not downloaded this particular message.
+      continue;
     }
 
     path->Append(NS_LITERAL_STRING("cur"));
     path->AppendNative(fileName);
 
-    // let's check if the folder exists
+    // Let's check if the message exists.
     bool exists;
     path->Exists(&exists);
+    if (!exists)
+    {
+      MOZ_LOG(MailDirLog, mozilla::LogLevel::Info,
+             ("DeleteMessages - file does not exist !!\n"));
+      // Perhaps an offline store has not downloaded this particular message.
+      continue;
+    }
     path->Remove(false);
   }
   return NS_OK;
@@ -857,23 +973,58 @@ nsMsgMaildirStore::CopyMessages(bool aIsMove, nsIArray *aHdrArray,
   NS_ENSURE_ARG_POINTER(aDstFolder);
   NS_ENSURE_ARG_POINTER(aCopyDone);
   NS_ENSURE_ARG_POINTER(aUndoAction);
-  PRUint32 messageCount;
-  nsresult rv = aHdrArray->GetLength(&messageCount);
-  NS_ENSURE_SUCCESS(rv, rv);
-  nsCOMPtr<nsIMsgFolder> srcFolder;
-  nsCOMPtr<nsIFile> destFolderPath;
-  nsCOMPtr<nsIMsgDatabase> destDB;
-  nsCOMPtr<nsIMsgDatabase> srcDB;
-  aDstFolder->GetMsgDatabase(getter_AddRefs(destDB));
-  aDstFolder->GetFilePath(getter_AddRefs(destFolderPath));
-  destFolderPath->Append(NS_LITERAL_STRING("cur"));
 
+  *aCopyDone = false;
+
+  nsCOMPtr<nsIMsgFolder> srcFolder;
+  nsresult rv;
   nsCOMPtr<nsIMsgDBHdr> msgHdr = do_QueryElementAt(aHdrArray, 0, &rv);
   NS_ENSURE_SUCCESS(rv, rv);
   rv = msgHdr->GetFolder(getter_AddRefs(srcFolder));
   NS_ENSURE_SUCCESS(rv, rv);
+
+  // Both source and destination folders must use maildir type store.
+  nsCOMPtr<nsIMsgPluggableStore> srcStore;
+  nsAutoCString srcType;
+  srcFolder->GetMsgStore(getter_AddRefs(srcStore));
+  if (srcStore)
+    srcStore->GetStoreType(srcType);
+  nsCOMPtr<nsIMsgPluggableStore> dstStore;
+  nsAutoCString dstType;
+  aDstFolder->GetMsgStore(getter_AddRefs(dstStore));
+  if (dstStore)
+    dstStore->GetStoreType(dstType);
+  if (!srcType.EqualsLiteral("maildir") || !dstType.EqualsLiteral("maildir"))
+    return NS_OK;
+
+  // Both source and destination must be local folders. In theory we could
+  //   do efficient copies of the offline store of IMAP, but this is not
+  //   supported yet. For that, we need to deal with both correct handling
+  //   of deletes from the src server, and msgKey = UIDL in the dst folder.
+  nsCOMPtr<nsIMsgLocalMailFolder> destLocalFolder(do_QueryInterface(aDstFolder));
+  if (!destLocalFolder)
+    return NS_OK;
+  nsCOMPtr<nsIMsgLocalMailFolder> srcLocalFolder(do_QueryInterface(srcFolder));
+  if (!srcLocalFolder)
+    return NS_OK;
+
+  // We should be able to use a file move for an efficient copy.
+
+  nsCOMPtr<nsIFile> destFolderPath;
+  nsCOMPtr<nsIMsgDatabase> destDB;
+  aDstFolder->GetMsgDatabase(getter_AddRefs(destDB));
+  rv = aDstFolder->GetFilePath(getter_AddRefs(destFolderPath));
+  NS_ENSURE_SUCCESS(rv, rv);
+  destFolderPath->Append(NS_LITERAL_STRING("cur"));
+
+  nsCOMPtr<nsIFile> srcFolderPath;
+  rv = srcFolder->GetFilePath(getter_AddRefs(srcFolderPath));
+  NS_ENSURE_SUCCESS(rv, rv);
+  srcFolderPath->Append(NS_LITERAL_STRING("cur"));
+
+  nsCOMPtr<nsIMsgDatabase> srcDB;
   srcFolder->GetMsgDatabase(getter_AddRefs(srcDB));
-  nsRefPtr<nsLocalMoveCopyMsgTxn> msgTxn = new nsLocalMoveCopyMsgTxn;
+  RefPtr<nsLocalMoveCopyMsgTxn> msgTxn = new nsLocalMoveCopyMsgTxn;
   NS_ENSURE_TRUE(msgTxn, NS_ERROR_OUT_OF_MEMORY);
   if (NS_SUCCEEDED(msgTxn->Init(srcFolder, aDstFolder, aIsMove)))
   {
@@ -883,34 +1034,40 @@ nsMsgMaildirStore::CopyMessages(bool aIsMove, nsIArray *aHdrArray,
       msgTxn->SetTransactionType(nsIMessenger::eCopyMsg);
   }
 
+  if (aListener)
+    aListener->OnStartCopy();
+
   nsCOMPtr<nsIMutableArray> dstHdrs(do_CreateInstance(NS_ARRAY_CONTRACTID, &rv));
   NS_ENSURE_SUCCESS(rv, rv);
-  for (PRUint32 i = 0; i < messageCount; i++)
-  {
-    nsCOMPtr<nsIMsgDBHdr> msgHdr = do_QueryElementAt(aHdrArray, i, &rv);
-    if (NS_FAILED(rv))
-      continue;
-    nsMsgKey srcKey;
-    msgHdr->GetMessageKey(&srcKey);
-    msgTxn->AddSrcKey(srcKey);
-    msgHdr->GetFolder(getter_AddRefs(srcFolder));
-    nsCOMPtr<nsIFile> path;
-    rv = srcFolder->GetFilePath(getter_AddRefs(path));
-    NS_ENSURE_SUCCESS(rv, rv);
-    nsCAutoString fileName;
-    msgHdr->GetStringProperty("storeToken", getter_Copies(fileName));
-    if (fileName.IsEmpty())
-      return NS_ERROR_FAILURE;
+  uint32_t messageCount;
+  rv = aHdrArray->GetLength(&messageCount);
+  NS_ENSURE_SUCCESS(rv, rv);
 
+  for (uint32_t i = 0; i < messageCount; i++)
+  {
+    nsCOMPtr<nsIMsgDBHdr> srcHdr = do_QueryElementAt(aHdrArray, i, &rv);
+    if (NS_FAILED(rv))
+    {
+      MOZ_LOG(MailDirLog, mozilla::LogLevel::Info,
+             ("srcHdr null\n"));
+      continue;
+    }
+    nsMsgKey srcKey;
+    srcHdr->GetMessageKey(&srcKey);
+    msgTxn->AddSrcKey(srcKey);
+    nsAutoCString fileName;
+    srcHdr->GetStringProperty("storeToken", getter_Copies(fileName));
     if (fileName.IsEmpty())
     {
-      PR_LOG(MailDirLog, PR_LOG_ALWAYS,
+      MOZ_LOG(MailDirLog, mozilla::LogLevel::Info,
              ("GetMsgInputStream - empty storeToken!!\n"));
       return NS_ERROR_FAILURE;
     }
 
-    path->Append(NS_LITERAL_STRING("cur"));
-    path->AppendNative(fileName);
+    nsCOMPtr<nsIFile> srcFile;
+    rv = srcFolderPath->Clone(getter_AddRefs(srcFile));
+    NS_ENSURE_SUCCESS(rv, rv);
+    srcFile->AppendNative(fileName);
 
     nsCOMPtr<nsIFile> destFile;
     destFolderPath->Clone(getter_AddRefs(destFile));
@@ -924,39 +1081,46 @@ nsMsgMaildirStore::CopyMessages(bool aIsMove, nsIArray *aHdrArray,
       destFile->GetNativeLeafName(fileName);
     }
     if (aIsMove)
-      path->MoveToNative(destFolderPath, fileName);
+      rv = srcFile->MoveToNative(destFolderPath, fileName);
     else
-      path->CopyToNative(destFolderPath, fileName);
+      rv = srcFile->CopyToNative(destFolderPath, fileName);
+    NS_ENSURE_SUCCESS(rv, rv);
 
     nsCOMPtr<nsIMsgDBHdr> destHdr;
     if (destDB)
     {
-      rv = destDB->CopyHdrFromExistingHdr(nsMsgKey_None, msgHdr, true, getter_AddRefs(destHdr));
+      rv = destDB->CopyHdrFromExistingHdr(nsMsgKey_None, srcHdr, true, getter_AddRefs(destHdr));
       NS_ENSURE_SUCCESS(rv, rv);
       destHdr->SetStringProperty("storeToken", fileName.get());
       dstHdrs->AppendElement(destHdr, false);
       nsMsgKey dstKey;
       destHdr->GetMessageKey(&dstKey);
       msgTxn->AddDstKey(dstKey);
+      if (aListener)
+        aListener->SetMessageKey(dstKey);
     }
   }
   nsCOMPtr<nsIMsgFolderNotificationService> notifier(do_GetService(NS_MSGNOTIFICATIONSERVICE_CONTRACTID));
   if (notifier)
     notifier->NotifyMsgsMoveCopyCompleted(aIsMove, aHdrArray, aDstFolder,
                                           dstHdrs);
+
+  // For now, we only support local dest folders, and for those we are done and
+  // can delete the messages. Perhaps this should be moved into the folder
+  // when we try to support other folder types.
   if (aIsMove)
   {
-    for (PRUint32 i = 0; i < messageCount; ++i)
+    for (uint32_t i = 0; i < messageCount; ++i)
     {
       nsCOMPtr<nsIMsgDBHdr> msgDBHdr(do_QueryElementAt(aHdrArray, i, &rv));
-      rv = srcDB->DeleteHeader(msgDBHdr, nsnull, false, true);
+      rv = srcDB->DeleteHeader(msgDBHdr, nullptr, false, true);
     }
   }
+
   *aCopyDone = true;
   nsCOMPtr<nsISupports> srcSupports(do_QueryInterface(srcFolder));
-  nsCOMPtr<nsIMsgLocalMailFolder> localDest(do_QueryInterface(aDstFolder));
-  if (localDest)
-    localDest->OnCopyCompleted(srcSupports, true);
+  if (destLocalFolder)
+    destLocalFolder->OnCopyCompleted(srcSupports, true);
   if (aListener)
     aListener->OnStopCopy(NS_OK);
   msgTxn.forget(aUndoAction);
@@ -1029,17 +1193,16 @@ nsresult MaildirStoreParser::ParseNextMessage(nsIFile *aFile)
   rv = NS_NewLocalFileInputStream(getter_AddRefs(inputStream), aFile);
   if (NS_SUCCEEDED(rv) && inputStream)
   {
-    PRInt32 inputBufferSize = 10240;
     nsMsgLineStreamBuffer *inputStreamBuffer =
-      new nsMsgLineStreamBuffer(inputBufferSize, true, false);
-    PRInt64 fileSize;
+      new nsMsgLineStreamBuffer(FILE_IO_BUFFER_SIZE, true, false);
+    int64_t fileSize;
     aFile->GetFileSize(&fileSize);
     msgParser->SetNewMsgHdr(newMsgHdr);
     msgParser->SetState(nsIMsgParseMailMsgState::ParseHeadersState);
     msgParser->SetEnvelopePos(0);
     bool needMoreData = false;
-    char * newLine = nsnull;
-    PRUint32 numBytesInLine = 0;
+    char * newLine = nullptr;
+    uint32_t numBytesInLine = 0;
     // we only have to read the headers, because we know the message size
     // from the file size. So we can do this in one time slice.
     do
@@ -1055,9 +1218,9 @@ nsresult MaildirStoreParser::ParseNextMessage(nsIFile *aFile)
 
     msgParser->FinishHeader();
     // A single message needs to be less than 4GB
-    newMsgHdr->SetMessageSize((PRUint32) fileSize);
+    newMsgHdr->SetMessageSize((uint32_t) fileSize);
     m_db->AddNewHdrToDB(newMsgHdr, true);
-    nsCAutoString storeToken;
+    nsAutoCString storeToken;
     aFile->GetNativeLeafName(storeToken);
     newMsgHdr->SetStringProperty("storeToken", storeToken.get());
   }
@@ -1086,19 +1249,21 @@ void MaildirStoreParser::TimerCallback(nsITimer *aTimer, void *aClosure)
       {
         nsCOMPtr<nsIMsgMailNewsUrl> url = do_QueryInterface(mailboxurl);
         url->SetUpdatingFolder(true);
-        nsCAutoString uriSpec("mailbox://");
+        nsAutoCString uriSpec("mailbox://");
         // ### TODO - what if SetSpec fails?
         (void) url->SetSpec(uriSpec);
         parser->m_listener->OnStopRunningUrl(url, NS_OK);
       }
     }
+    // Parsing complete and timer cancelled, so we release the parser object.
+    delete parser;
     return;
   }
   nsCOMPtr<nsISupports> aSupport;
   parser->m_directoryEnumerator->GetNext(getter_AddRefs(aSupport));
   nsresult rv;
   nsCOMPtr<nsIFile> currentFile(do_QueryInterface(aSupport, &rv));
-  NS_ENSURE_SUCCESS(rv,);
+  NS_ENSURE_SUCCESS_VOID(rv);
   parser->ParseNextMessage(currentFile);
   // ### TODO - what if this fails?
 }
@@ -1135,20 +1300,21 @@ NS_IMETHODIMP nsMsgMaildirStore::RebuildIndex(nsIMsgFolder *aFolder,
                                                           aListener);
   NS_ENSURE_TRUE(fileParser, NS_ERROR_OUT_OF_MEMORY);
   fileParser->StartTimer();
+  ResetForceReparse(aMsgDB);
   return NS_OK;
 }
 
 NS_IMETHODIMP nsMsgMaildirStore::ChangeFlags(nsIArray *aHdrArray,
-                                             PRUint32 aFlags,
+                                             uint32_t aFlags,
                                              bool aSet)
 {
   NS_ENSURE_ARG_POINTER(aHdrArray);
 
-  PRUint32 messageCount;
+  uint32_t messageCount;
   nsresult rv = aHdrArray->GetLength(&messageCount);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  for (PRUint32 i = 0; i < messageCount; i++)
+  for (uint32_t i = 0; i < messageCount; i++)
   {
     nsCOMPtr<nsIMsgDBHdr> msgHdr = do_QueryElementAt(aHdrArray, i, &rv);
     // get output stream for header
@@ -1169,7 +1335,7 @@ nsMsgMaildirStore::GetOutputStream(nsIMsgDBHdr *aHdr,
                                    nsCOMPtr<nsIOutputStream> &aOutputStream)
 {
   // file name is stored in message header property "storeToken"
-  nsCAutoString fileName;
+  nsAutoCString fileName;
   aHdr->GetStringProperty("storeToken", getter_Copies(fileName));
   if (fileName.IsEmpty())
     return NS_ERROR_FAILURE;
@@ -1199,20 +1365,19 @@ NS_IMETHODIMP nsMsgMaildirStore::ChangeKeywords(nsIArray *aHdrArray,
   nsCOMPtr<nsIOutputStream> outputStream;
   nsCOMPtr<nsISeekableStream> seekableStream;
 
-  PRUint32 messageCount;
+  uint32_t messageCount;
   nsresult rv = aHdrArray->GetLength(&messageCount);
   NS_ENSURE_SUCCESS(rv, rv);
   if (!messageCount)
     return NS_ERROR_INVALID_ARG;
 
-  nsLineBuffer<char> *lineBuffer;
-  rv = NS_InitLineBuffer(&lineBuffer);
-  NS_ENSURE_SUCCESS(rv, rv);
+  nsAutoPtr<nsLineBuffer<char> > lineBuffer(new nsLineBuffer<char>);
+  NS_ENSURE_TRUE(lineBuffer, NS_ERROR_OUT_OF_MEMORY);
 
   nsTArray<nsCString> keywordArray;
   ParseString(aKeywords, ' ', keywordArray);
 
-  for (PRUint32 i = 0; i < messageCount; ++i) // for each message
+  for (uint32_t i = 0; i < messageCount; ++i) // for each message
   {
     nsCOMPtr<nsIMsgDBHdr> message = do_QueryElementAt(aHdrArray, i, &rv);
     NS_ENSURE_SUCCESS(rv, rv);
@@ -1224,9 +1389,9 @@ NS_IMETHODIMP nsMsgMaildirStore::ChangeKeywords(nsIArray *aHdrArray,
     NS_ENSURE_SUCCESS(rv, rv);
     nsCOMPtr <nsISeekableStream> seekableStream(do_QueryInterface(inputStream, &rv));
     NS_ENSURE_SUCCESS(rv, rv);
-    PRUint32 statusOffset = 0;
+    uint32_t statusOffset = 0;
     (void)message->GetStatusOffset(&statusOffset);
-    PRUint64 desiredOffset = statusOffset;
+    uint64_t desiredOffset = statusOffset;
 
     ChangeKeywordsHelper(message, desiredOffset, lineBuffer, keywordArray,
                          aAdd, outputStream, seekableStream, inputStream);
@@ -1236,7 +1401,13 @@ NS_IMETHODIMP nsMsgMaildirStore::ChangeKeywords(nsIArray *aHdrArray,
     // we need to rewrite the message file with extra room for the keywords,
     // or schedule some sort of background task to do this.
   }
-  PR_Free(lineBuffer);
+  lineBuffer = nullptr;
+  return NS_OK;
+}
+
+NS_IMETHODIMP nsMsgMaildirStore::GetStoreType(nsACString& aType)
+{
+  aType.AssignLiteral("maildir");
   return NS_OK;
 }
 
